.MODULE/RAM/BOOT=0           dpiir_2p2z_32_16; 
 
{ 
 
         IIR Filter of the Form: 
 
              y(n)=b0*x(n)+b1*x(n-1)+b2*x(n-2)+a1*y(n-1)+a2*x(n-2) 
 
         Where: 
 
              x(n), x(n-1), x(n-2), y(n-1), y(n-2) are 32-bits 
              b0, b1, b2, a1, a2 are 16-bits 
 
         Calling Parameters: 
 
              AX0=least significant word (LSW) of input 
              AX1=most significant word (MSW) of input 
              I0=start address of delay line in data memory stored in 
                 LSW,MSW order [organized x(n-1),x(n-2),y(n-1),y(n-2)] 
              I4=start address of coefficients in program memory 
                 [organized b0,b1,b2,a1,a2] 
              I5=start address of coefficient scaling factors in program 
                 memory (one/section) 
              L0=(4*number of sections)+4 
              L4=5*number of sections 
              L5=number of sections 
              M0=-1, M1=1, M2=2, M3=-5 
              M4=-1, M5=1, M6=2, M7=-4 
              CNTR=number of sections 
 
         Return Values: 
 
              I0=new start address of delay line 
              SR1=MSW result 
              SR0=LSW result 
 
         Altered Registers: 
 
              MX0,MY0,MR,SE,SR 
 
         Computation Time: 
 
              9 + (16 * number of sections) 
 
         Coefficients in 1.15 (Q.15) format 
         Delay line, input sample in 1.31 (Q.31) format 
 
} 
 
 
.ENTRY                       biq_3216; 
 
 
biq_3216:SR1=AX1; 
         SR0=AX0; 
         MY0=PM(I4,M5); 
         SE=PM(I5,M5); 
 
         DO biq UNTIL CE; 
 
              MR=SR0*MY0(US), MX0=DM(I0,M2), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(US), MX0=DM(I0,M2), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(US), MX0=DM(I0,M2), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(US), MX0=DM(I0,M3), MY0=PM(I4,M7); 
              MR=MR+MX0*MY0(US), MY0=PM(I4,M5); 
 
              MR0=MR1; 
              MR1=MR2; 
 
              MR=MR+SR1*MY0(SS), MX0=DM(I0,M2), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(SS), MX0=DM(I0,M2), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(SS), MX0=DM(I0,M2), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(SS), MX0=DM(I0,M3), MY0=PM(I4,M5); 
              MR=MR+MX0*MY0(SS); 
 
              SR=ASHIFT MR1 (HI), MY0=PM(I4,M5); 
              SR=SR OR LSHIFT MR0 (LO), SE=PM(I5,M5); 
 
              DM(I0,M1)=SR0; 
biq:          DM(I0,M1)=SR1; 
 
         MODIFY(I0,M2); 
         DM(I0,M1)=AX0; 
         DM(I0,M0)=AX1; 
 
         RTS; 
 
.ENDMOD;                                            